@@ -9,7 +9,7 @@ from GoogleScraper.utils import grouper
 from GoogleScraper.database import ScraperSearch, SERP, Link, get_session
 from GoogleScraper.proxies import parse_proxy_file, get_proxies_from_mysql_db
 from GoogleScraper.scraping import SelScrape, HttpScrape
-from GoogleScraper.caching import maybe_clean_cache, fix_broken_cache_names, _caching_is_one_to_one, parse_all_cached_files
+from GoogleScraper.caching import maybe_clean_cache, fix_broken_cache_names, _caching_is_one_to_one, parse_all_cached_files, clean_cachefiles
 from GoogleScraper.config import InvalidConfigurationException, parse_cmd_args, Config
 import GoogleScraper.config
 
@@ -32,7 +32,7 @@ def scrape_with_config(config, **kwargs):
     return main(return_results=True, parse_cmd_line=False, **kwargs)
 
 def assign_keywords_to_scrapers(all_keywords):
-    """Scrapers are often threads or asynchronous objects.
+    """Scrapers are threads or asynchronous objects.
 
     Splitting the keywords equally on the workers is crucial
     for maximal performance.
@@ -41,7 +41,7 @@ def assign_keywords_to_scrapers(all_keywords):
         all_keywords: All keywords to scrape
 
     Returns:
-        A list of list. The inner list should be assigned to individual scrapers.
+        A list of list. The elements of the inner list should be assigned to individual scraper instances, e. g. threads.
     """
     mode = Config['SCRAPING'].get('scrapemethod')
 
@@ -117,6 +117,11 @@ def main(return_results=False, parse_cmd_line=True):
         print(open(CONFIG_FILE).read())
         return
 
+    if Config['GLOBAL'].getboolean('version'):
+        from GoogleScraper.version import __version__
+        print(__version__)
+        return
+
     maybe_clean_cache()
 
     kwfile = Config['SCRAPING'].get('keyword_file')
@@ -160,6 +165,10 @@ def main(return_results=False, parse_cmd_line=True):
     search_engines = list({search_engine for search_engine in Config['SCRAPING'].get('search_engines', 'google').split(',') if search_engine})
     assert search_engines, 'No search engine specified'
 
+    if Config['GLOBAL'].getboolean('clean_cache_files', False):
+        clean_cachefiles()
+        return
+
     if Config['GLOBAL'].getboolean('check_oto', False):
         _caching_is_one_to_one(keyword)
 
@@ -175,7 +184,8 @@ def main(return_results=False, parse_cmd_line=True):
 
     if Config['SCRAPING'].getboolean('use_own_ip'):
         proxies.append(None)
-    elif not proxies:
+        
+    if not proxies:
         raise InvalidConfigurationException("No proxies available and using own IP is prohibited by configuration. Turning down.")
 
     valid_search_types = ('normal', 'video', 'news', 'image')
@@ -219,7 +229,6 @@ def main(return_results=False, parse_cmd_line=True):
     remaining = [keyword for keyword in set(remaining) if keyword]
 
     kwgroups = assign_keywords_to_scrapers(remaining)
-    chunks_per_proxy = math.ceil(len(kwgroups)/len(proxies))
 
     # Create a lock to synchronize database access in the sqlalchemy session
     db_lock = threading.Lock()
@@ -229,7 +238,7 @@ def main(return_results=False, parse_cmd_line=True):
 
     # final check before going into the loop
     num_workers_to_allocate = len(kwgroups) * len(search_engines) > Config['SCRAPING'].getint('maximum_workers')
-    if (len(kwgroups) * len(search_engines))  > Config['SCRAPING'].getint('maximum_workers'):
+    if (len(kwgroups) * len(search_engines)) > Config['SCRAPING'].getint('maximum_workers'):
         logger.error('Too many workers: {} , might crash the app'.format(num_workers_to_allocate))
 
     # Let the games begin
@@ -242,7 +251,9 @@ def main(return_results=False, parse_cmd_line=True):
 
         for k, search_engine in enumerate(search_engines):
             for i, keyword_group in enumerate(kwgroups):
-                proxy_to_use = proxies[i//chunks_per_proxy]
+                
+                proxy_to_use = proxies[i % len(proxies)]
+                
                 if Config['SCRAPING'].get('scrapemethod', 'http') == 'selenium':
                     scrapejobs.append(
                         SelScrape(
@@ -287,4 +298,4 @@ def main(return_results=False, parse_cmd_line=True):
     session.commit()
 
     if return_results:
-        return session
\ No newline at end of file
+        return session
