@@ -5,7 +5,9 @@
 import sys
 import re
 import lxml.html
+from lxml.html.clean import Cleaner
 import logging
+from urllib.parse import urlparse
 import pprint
 from GoogleScraper.database import SearchEngineResultsPage, Link
 from cssselect import HTMLTranslator, SelectorError
@@ -96,9 +98,16 @@ class Parser():
         """
         
         # Try to parse the provided HTML string using lxml
+        # strip all unnecessary information to save space
+        cleaner = Cleaner()
+        cleaner.scripts = True
+        cleaner.javascript = True
+        cleaner.style = True
+
         try:
             parser = lxml.html.HTMLParser(encoding='utf-8')
             self.dom = lxml.html.document_fromstring(self.html, parser=parser)
+            self.dom = cleaner.clean_html(self.dom)
             self.dom.resolve_base_href()
         except Exception as e:
             # maybe wrong encoding
@@ -172,25 +181,6 @@ class Parser():
                         serp_result[key] = value
                     if serp_result:
                         self.search_results[result_type].append(serp_result)
-
-    def clean_html(self, html):
-        """Clean the html from any bloated data that is of no interest.
-
-        For example in Google SERP html pages, most data is lots of
-        javascript code that tells os nothing about the results. When
-        stripping it away and then caching, we save a lot of disk space.
-
-        Args:
-            html: The html to clean.
-
-        Returns:
-            The cleaned html.
-        """
-        parsed = lxml.html.fromstring(html)
-        for bad in parsed.xpath('//script|//style'):
-            bad.getparent().remove(bad)
-
-        return lxml.html.tostring(parsed)
                     
     def after_parsing(self):
         """Subclass specific behaviour after parsing happened.
@@ -202,6 +192,11 @@ class Parser():
     def __str__(self):
         """Return a nicely formated overview of the results."""
         return pprint.pformat(self.search_results)
+
+    @property
+    def cleaned_html(self):
+        assert self.dom, 'The html needs to be parsed to get the cleaned html'
+        return lxml.html.tostring(self.dom)
                 
 """
 Here follow the different classes that provide CSS selectors 
@@ -502,7 +497,7 @@ def get_parser_by_search_engine(search_engine):
 
 def parse_serp(html=None, search_engine=None,
                     scrapemethod=None, current_page=None, requested_at=None,
-                    requested_by='127.0.0.1', current_keyword=None):
+                    requested_by='127.0.0.1', current_keyword=None, parser=None, serp=None):
         """Store the parsed data in the sqlalchemy session.
 
         Args:
@@ -512,31 +507,35 @@ def parse_serp(html=None, search_engine=None,
             The parsed SERP object.
         """
 
-        parser = get_parser_by_search_engine(search_engine)
-        parser = parser()
-        parser.parse(html)
+        if not parser:
+            parser = get_parser_by_search_engine(search_engine)
+            parser = parser()
+            parser.parse(html)
 
         num_results = 0
 
-        serp = SearchEngineResultsPage(
-            search_engine_name=search_engine,
-            scrapemethod=scrapemethod,
-            page_number=current_page,
-            requested_at=requested_at,
-            requested_by=requested_by,
-            query=current_keyword,
-            num_results_for_keyword=parser.search_results['num_results'],
-        )
+        if not serp:
+            serp = SearchEngineResultsPage(
+                search_engine_name=search_engine,
+                scrapemethod=scrapemethod,
+                page_number=current_page,
+                requested_at=requested_at,
+                requested_by=requested_by,
+                query=current_keyword,
+                num_results_for_keyword=parser.search_results['num_results'],
+            )
 
         for key, value in parser.search_results.items():
             if isinstance(value, list):
                 rank = 1
                 for link in value:
+                    parsed = urlparse(link['link'])
                     l = Link(
-                        url=link['link'],
+                        link=link['link'],
                         snippet=link['snippet'],
                         title=link['title'],
                         visible_link=link['visible_link'],
+                        domain=parsed.netloc,
                         rank=rank,
                         serp=serp
                     )
