@@ -3,6 +3,8 @@
 import threading
 import datetime
 import random
+import json
+import csv
 import math
 import logging
 import sys
@@ -22,11 +24,12 @@ except ImportError as ie:
     sys.exit('You can install missing modules with `pip3 install [modulename]`')
 
 import GoogleScraper.socks as socks
+from GoogleScraper.proxies import Proxy
 from GoogleScraper.caching import get_cached, cache_results, cached_file_name, cached
 from GoogleScraper.database import SearchEngineResultsPage, Link, get_session
 from GoogleScraper.config import Config
 from GoogleScraper.log import out
-from GoogleScraper.parsing import GoogleParser, YahooParser, YandexParser, BaiduParser, BingParser, DuckduckgoParser, get_parser_by_search_engine
+from GoogleScraper.parsing import GoogleParser, YahooParser, YandexParser, BaiduParser, BingParser, DuckduckgoParser, get_parser_by_search_engine, parse_serp
 
 logger = logging.getLogger('GoogleScraper')
 
@@ -136,6 +139,9 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
             self.keywords = Config['SCRAPING'].get('keywords', [])
 
         self.keywords = list(set(self.keywords))
+
+        # the number of keywords
+        self.num_keywords = len(self.keywords)
         
         # The actual keyword that is to be scraped next
         self.current_keyword = self.keywords[0]
@@ -157,11 +163,11 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
         
         # Install the proxy if one was provided
         self.proxy = proxy
-        if proxy:
+        if isinstance(proxy, Proxy):
             self.set_proxy()
             self.ip = self.proxy.host + ':' + self.proxy.port
         else:
-            self.ip = '127.0.0.1'
+            self.ip = 'localhost'
 
         # the scraper_search object
         self.scraper_search = scraper_search
@@ -179,6 +185,9 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
         # set the session
         self.session = session
 
+        # the current request time
+        self.current_request_time = None
+
 
     @abc.abstractmethod
     def search(self, *args, **kwargs):
@@ -197,8 +206,6 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
         """
         for self.current_keyword in self.keywords:
 
-            out('Next Keyword="{kw}" requested by {scraper} and ip {ip}'.format(kw=self.current_keyword, scraper=self.__class__.__name__, ip=self.ip), lvl=2)
-
             self.current_page = self.start_page_pos
 
             for self.current_page in range(1, self.num_pages_per_keyword + 1):
@@ -236,37 +243,84 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
                 search_engine_name=self.search_engine,
                 scrapemethod=self.scrapemethod,
                 page_number=self.current_page,
-                requested_at=datetime.datetime.utcnow(),
+                requested_at=self.current_request_time,
                 requested_by=self.ip,
                 query=self.current_keyword,
                 num_results_for_keyword=self.parser.search_results['num_results'],
             )
             self.scraper_search.serps.append(serp)
 
-            for key, value in self.parser.search_results.items():
-                if isinstance(value, list):
-                    rank = 1
-                    for link in value:
-                        l = Link(
-                            url=link['link'],
-                            snippet=link['snippet'],
-                            title=link['title'],
-                            visible_link=link['visible_link'],
-                            rank=rank,
-                            serp=serp
-                        )
-                        self.session.add(l)
-                        num_results += 1
-                        rank += 1
-
-            serp.num_results = num_results
+            parse_serp(serp=serp, parser=self.parser)
             self.session.add(serp)
             self.session.commit()
 
+        output_format = Config['GLOBAL'].get('output_format', 'stdout')
+        output_file = Config['GLOBAL'].get('output_filename', 'google_scraper')
+
+        if output_format == 'stdout':
+            out(self.parser, lvl=2)
+        elif output_format == 'json':
+            if not hasattr(self, 'json_outfile'):
+                self.json_outfile = open(output_file + '.json', 'a')
+
+            obj = self._get_serp_obj()
+            json.dump(obj, self.json_outfile, indent=2, sort_keys=True)
+
+        elif output_format == 'csv':
+            if not hasattr(self, 'csv_outfile'):
+                self.csv_outfile = csv.DictWriter(open(output_file + '.csv', 'a'),
+                        fieldnames=('link', 'title', 'snippet', 'visible_link', 'num_results',
+                                    'query', 'search_engine_name', 'requested_by',
+                                    'scrapemethod', 'page_number', 'requested_at'))
+                self.csv_outfile.writeheader()
+
+            rows = []
+            for result_type, value in self.parser.search_results.items():
+                if isinstance(value, list):
+                    for link in value:
+                        rows.append(link)
+
+            obj = self._get_serp_obj()
+            obj['num_results'] = self.parser.search_results['num_results']
+            for row in rows:
+                row.update(obj)
+                self.csv_outfile.writerow(row)
+
+
+    def _get_serp_obj(self):
+        """Little helper that returns a serp object for various output formats."""
+        obj = {}
+        obj['query'] = self.current_keyword
+        obj['search_engine_name'] = self.search_engine
+        obj['requested_by'] = self.ip
+        obj['scrapemethod'] = self.scrapemethod
+        obj['page_number'] = self.current_page
+        obj['requested_at'] = self.current_request_time
+        return obj
+
     def next_page(self):
         """Increment the page. The next search request will request the next page."""
         self.start_page_pos += 1
 
+    def next_keyword_info(self, pos_current_keyword):
+        """Print a short summary where we are in the scrape and what's the next keyword."""
+        out('[{thread_name}][{search_engine} with ip {ip} next keyword: "{keyword}" with {num_pages} pages. {done}/{all} already scraped.'.format(
+            thread_name=self.name,
+            search_engine=self.search_engine,
+            ip=self.ip,
+            keyword=self.current_keyword,
+            num_pages=self.num_pages_per_keyword,
+            done=pos_current_keyword,
+            all=self.num_keywords
+        ), lvl=1)
+
+
+    def cache_results(self):
+        """Caches the html for the current request."""
+        if Config['GLOBAL'].getboolean('do_caching', False):
+            with self.cache_lock:
+                cache_results(self.parser.cleaned_html, self.current_keyword, self.search_engine, self.scrapemethod)
+
 
 class HttpScrape(SearchEngineScrape, threading.Timer):
     """Offers a fast way to query any search engine using raw HTTP requests.
@@ -458,46 +512,45 @@ class HttpScrape(SearchEngineScrape, threading.Timer):
         if rand:
             self.headers['User-Agent'] = random.choice(self.USER_AGENTS)
 
-        html = get_cached(self.current_keyword, self.search_engine, 'http')
+        try:
+            out('[HTTP - {proxy}] Base_url: {base_url}, headers={headers}, params={params}'.format(
+                proxy=self.proxy,
+                base_url=self.base_search_url,
+                headers=self.headers,
+                params=self.search_params),
+            lvl=3)
 
-        if not html:
-            try:
-                out('[HTTP - {proxy}] Base_url: {base_url}, headers={headers}, params={params}'.format(
-                    proxy=self.proxy,
-                    base_url=self.base_search_url,
-                    headers=self.headers,
-                    params=self.search_params),
-                lvl=3)
+            super().next_keyword_info(self.n)
 
-                request = self.requests.get(self.base_search_url, headers=self.headers,
-                                 params=self.search_params, timeout=3.0)
+            request = self.requests.get(self.base_search_url, headers=self.headers,
+                             params=self.search_params, timeout=3.0)
 
-            except self.requests.ConnectionError as ce:
-                logger.error('Network problem occurred {}'.format(ce))
-                raise ce
-            except self.requests.Timeout as te:
-                logger.error('Connection timeout {}'.format(te))
-                raise te
+            self.current_request_time = datetime.datetime.utcnow()
 
-            if not request.ok:
-                logger.error('HTTP Error: {}'.format(request.status_code))
-                self.handle_request_denied(request.status_code)
-                return False
+        except self.requests.ConnectionError as ce:
+            logger.error('Network problem occurred {}'.format(ce))
+            raise ce
+        except self.requests.Timeout as te:
+            logger.error('Connection timeout {}'.format(te))
+            raise te
 
-            html = request.text
-
-            # cache fresh results
-            with self.cache_lock:
-                cache_results(html, self.current_keyword, self.search_engine, self.scrapemethod)
+        if not request.ok:
+            logger.error('HTTP Error: {}'.format(request.status_code))
+            self.handle_request_denied(request.status_code)
+            return False
 
+        html = request.text
         self.parser.parse(html)
         self.store()
-        out(str(self.parser), lvl=2)
+        super().cache_results()
+
+        self.n += 1
 
     def run(self):
         args = []
         kwargs = {}
         kwargs['rand'] = False
+        self.n = 0
         SearchEngineScrape.blocking_search(self, self.search, *args, **kwargs)
         
 class AsyncHttpScrape(SearchEngineScrape):
@@ -654,38 +707,51 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         """Checks whether Google detected a potentially harmful request.
 
         Whenever such potential abuse is detected, Google shows an captcha.
+        This method just blocks as long as someone entered the captcha in the browser window.
+        When the window is not visible (For example when using PhantomJS), this method
+        makes a png from the html code and shows it to the user, which should enter it in a command
+        line.
 
         Returns:
-            True If the issue could be resolved.
+            The search input field.
 
         Raises:
             MaliciousRequestDetected when there was not way to stp Google From denying our requests.
         """
-        if Config['SELENIUM'].getboolean('manual_captcha_solving'):
-            if '/sorry/' in self.webdriver.current_url \
-                    and 'detected unusual traffic' in self.webdriver.page_source:
-                if Config['SELENIUM'].get('manual_captcha_solving', False):
-                    with self.captcha_lock:
-                        import tempfile
-                        tf = tempfile.NamedTemporaryFile('wb')
-                        tf.write(self.webdriver.get_screenshot_as_png())
-                        import webbrowser
-                        webbrowser.open('file://{}'.format(tf.name))
-                        solution = input('enter the captcha please...')
-                        self.webdriver.find_element_by_name('submit').send_keys(solution + Keys.ENTER)
-                        try:
-                            self.search_input = WebDriverWait(self.webdriver, 5).until(EC.presence_of_element_located((By.NAME, "q")))
-                        except TimeoutException as e:
-                            raise MaliciousRequestDetected('Requesting with this ip is not possible at the moment.')
-                        tf.close()
-                        return True
-            elif 'is not an HTTP Proxy' in self.webdriver.page_source:
-                raise GoogleSearchError('Inavlid TOR usage. Specify the proxy protocol as socks5')
-            else:
-                return False
-        else:
-            raise MaliciousRequestDetected('Requesting with this ip is not possible at the moment.')
+        malicious_request_needles = {
+            'google': {
+                'inurl': '/sorry/',
+                'inhtml': 'detected unusual traffic'
+            }
+        }
 
+        needles = malicious_request_needles[self.search_engine]
+
+        if needles['inurl'] in self.webdriver.current_url and needles['inhtml'] in self.webdriver.page_source:
+
+            if Config['SELENIUM'].getboolean('manual_captcha_solving', False):
+                with self.captcha_lock:
+                    import tempfile
+                    tf = tempfile.NamedTemporaryFile('wb')
+                    tf.write(self.webdriver.get_screenshot_as_png())
+                    import webbrowser
+                    webbrowser.open('file://{}'.format(tf.name))
+                    solution = input('enter the captcha please...')
+                    self.webdriver.find_element_by_name('submit').send_keys(solution + Keys.ENTER)
+                    try:
+                        self.search_input = WebDriverWait(self.webdriver, 5).until(self._get_search_input_field())
+                    except TimeoutException as e:
+                        raise MaliciousRequestDetected('Requesting with this ip is not possible at the moment.')
+                    tf.close()
+
+            else:
+                # Just wait until the user solves the captcha in the browser window
+                # 10 hours if needed :D
+                out('Waiting for user to solve captcha', lvl=1)
+                return self._wait_until_search_input_field_appears(10*60*60)
+
+        elif 'is not an HTTP Proxy' in self.webdriver.page_source:
+            raise GoogleSearchError('Inavlid TOR usage. Specify the proxy protocol as socks5')
 
     def build_search(self):
         """Build the search for SelScrapers"""
@@ -726,6 +792,21 @@ class SelScrape(SearchEngineScrape, threading.Thread):
 
         return input_field_selectors[self.search_engine]
 
+    def _wait_until_search_input_field_appears(self, max_wait=5):
+        """Waits until the search input field can be located for the current search engine
+
+        Args:
+            max_wait: How long to wait maximally before returning False.
+
+        Returns: False if the search input field could not be located within the time
+                or the handle to the search input field.
+        """
+        try:
+            search_input = WebDriverWait(self.webdriver, max_wait).until(
+                EC.visibility_of_element_located(self._get_search_input_field()))
+            return search_input
+        except TimeoutException as e:
+            return False
 
     def _goto_next_page(self):
         """Finds the url that locates the next page for any search_engine.
@@ -753,6 +834,7 @@ class SelScrape(SearchEngineScrape, threading.Thread):
             element = self.webdriver.find_element_by_css_selector(selector)
             next_url = element.get_attribute('href')
             element.click()
+            self.current_request_time = datetime.datetime.utcnow()
         except TimeoutException as te:
             logger.warning('Cannot locate next page element: {}'.format(te))
             return False
@@ -768,21 +850,24 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         Fills out the search form of the search engine for each keyword.
         Clicks the next link while num_pages_per_keyword is not reached.
         """
+        n = 0
+
         for self.current_keyword in self.keywords:
 
-            try:
-                self.search_input = WebDriverWait(self.webdriver, 5).until(
-                    EC.presence_of_element_located(self._get_search_input_field()))
-            except TimeoutException as e:
-                logger.error(e)
-                if not self.handle_request_denied():
-                    open('/tmp/out.png', 'wb').write(self.webdriver.get_screenshot_as_png())
-                    raise GoogleSearchError('search input field cannot be found.')
+            super().next_keyword_info(n)
+
+            self.search_input = self._wait_until_search_input_field_appears()
+
+            if self.search_input is False:
+                self.search_input = self.handle_request_denied()
 
             if self.search_input:
                 self.search_input.clear()
                 time.sleep(.25)
                 self.search_input.send_keys(self.current_keyword + Keys.ENTER)
+                self.current_request_time = datetime.datetime.utcnow()
+            else:
+                raise GoogleSearchError('Cannot get handle to the input form!')
 
             for self.current_page in range(1, self.num_pages_per_keyword + 1):
                 # Waiting until the keyword appears in the title may
@@ -791,20 +876,16 @@ class SelScrape(SearchEngineScrape, threading.Thread):
                     WebDriverWait(self.webdriver, 5).until(EC.title_contains(self.current_keyword))
                 except TimeoutException as e:
                     logger.error(SeleniumSearchError('Keyword "{}" not found in title: {}'.format(self.current_keyword, self.webdriver.title)))
+                    break
 
                 # match the largest sleep range
                 sleep_time = random.randrange(*self._largest_sleep_range(self.search_number))
                 time.sleep(sleep_time)
 
                 html = self.webdriver.page_source
-
                 self.parser.parse(html)
                 self.store()
-                out(str(self.parser), lvl=2)
-
-                # Lock for the sake that two threads write to same file (not probable)
-                with self.cache_lock:
-                    cache_results(html, self.current_keyword, self.search_engine, self.scrapemethod)
+                super().cache_results()
 
                 self.search_number += 1
 
@@ -815,6 +896,8 @@ class SelScrape(SearchEngineScrape, threading.Thread):
                     if not self.next_url:
                         break
 
+            n += 1
+
 
     def run(self):
         """Run the SelScraper."""
@@ -831,4 +914,4 @@ class SelScrape(SearchEngineScrape, threading.Thread):
 
         self.search()
 
-        self.webdriver.close()
\ No newline at end of file
+        self.webdriver.close()
